--[[***************************************************
        Config and Framework 配置和游戏框架
-- ***************************************************]]

require("config")
require("framework.init")
require("framework.shortcodes")
require("framework.cc.init")

--[[***************************************************
		Global Define 全局对象，资源，自定义基础类库
-- ***************************************************]]
require("app.datas.Resoures")
require("app.datas.Constant")
require("app.utils.Utils")

Global = require("framework.api.Context").new()
Setting = require("app.datas.Setting")
EffectHelper = require("app.sprites.EffectHelper")

--like NotifcationCenter
EventProtocol = require("framework.api.EventProtocol")
DataCenter = {}
DataCenter = EventProtocol.extend(DataCenter)

--[[***************************************************
        Systems Loading 游戏系统自动加载
-- ***************************************************]]

require("app.systems.SystemLoader")
SystemLoader.load()

--[[***************************************************
		GameManager 游戏逻辑
-- ***************************************************]]

local MyApp = class("MyApp", cc.mvc.AppBase)

local function clearMem()
    collectgarbage("collect")
end

function MyApp:ctor()
    MyApp.super.ctor(self)
end

function MyApp:run()
    CCDirector:sharedDirector():setProjection(kCCDirectorProjection2D)
	Setting:loadSetting()
    CCFileUtils:sharedFileUtils():addSearchPath("res/")

    if Setting:getSetting("FIRST_GUIDE") then
        self:goLogo()
    else
        self:goStart() 
    end    
end

function MyApp:goLogo(args)
    self:enterScene("LogoScene",args,"CCTransitionFade",0.3)
end

function MyApp:goStart(args)
    self:enterScene("StartScene",args,"CCTransitionFade",0.3)
end

function MyApp:goToGame(args)
    clearMem()
    self:enterScene("GameScene",args,"CCTransitionFade",0.8)
end

function MyApp:goToGold(args)
    local GoldScene = require("app.scenes.GoldScene")
    local scene = GoldScene.new(unpack(totable(args)))
    CCDirector:sharedDirector():pushScene(CCTransitionShrinkGrow:create(0.3,scene))
end

function MyApp:popGold(args)
    local director = CCDirector:sharedDirector()
    local scene = director:previousScene()
    CCDirector:sharedDirector():popScene(CCTransitionShrinkGrow:create(0.3,scene))
end

function MyApp:goToScore(args)
    self:enterScene("ScoreScene", args, "CCTransitionFade",0.5)
    clearMem()
end

function MyApp:gotoRank()
    self:enterScene("RankScene", args, "CCTransitionFade",0.5)
end

function MyApp:goToAchievement(args)
    self:enterScene("AchievementScene",args,"CCTransitionFade",0.5)
end

function MyApp:goToRecord(args)
    self:enterScene("RecordScene",args,"CCTransitionFade",0.3)
end

function MyApp:goToEquipment(args)
    self:enterScene("EquipmentScene",args,"CCTransitionFade",0.3)
end

function MyApp:goLoading(args)
	self:enterScene("LoadingScene",args,"CCTransitionFade",0.3)
end

function MyApp:goToForeverMode(args)
    self:enterScene("ForeverModeScene",args,"CCTransitionFade",0.3)
end

function MyApp:exit()
	Setting:saveSetting()
	os.exit()
end

return MyApp