--*******************************************
-- Desc: 是否复活层
-- File: ContinueLayer.lua
-- Author: zhouhaifeng
-- Date: 2014-03-09 15:44:01
--*******************************************

local TaskInfoCell = import("app.items.TaskInfoCell")
local StarRate = import("..utils.ui.StarRate")
local BuyConfirmLayer = import("app.layers.BuyConfirmLayer")
local PackageLayer = import("app.layers.PackageLayer")
local itemId = 9006

local ContinueLayer = class("ContinueLayer",function()
	return display.newColorLayer(ccc4(0,0,0,220))
end)

--[[
	初始化
	params: 地图信息
]]
function ContinueLayer:ctor()
    self.handler = nil

    local controlLayer = Global:get("controlLayer")
    local dead = controlLayer.record.numberOfDeads or 0
    local money = math.pow(2,dead) * 500 
    
    self.time = 9
    self.layer = UIModalLayer:create()
    self:addChild(self.layer);

    self.continueTitle = ccs.image({
        image = "ui/gameui/ui_continue_title.png",
        x = display.cx,
        y = display.cy + 140
    })


    self.timeLabel = ccs.labelAtlas({
        text = self.time,
        start = '1',
        image = "ui/gameui/number_bar_continue.png",
        w = 76,
        h = 85,
        x = display.cx,
        y = display.cy + 50,
    })

   self.hpStar = StarRate.new()
   self.hpStar:setup({ 
        max = 3,
        cur = EquipSystem:getPropNum(4000),
        on = "ui/common/hp_heart_n.png",
        off = "ui/common/hp_heart_h.png",
        x = display.left + 50 ,
        y = display.top - 50,   
    })
   self.hpStar:setScale(0.75)
   self:addChild(self.hpStar)

    self.resumeButton = ccs.button({
        normal  = "ui/buy/btn_buy_n.png",
        pressed = "ui/buy/btn_buy_h.png",
        x = display.cx + 140,
        y = display.bottom + 80,
        text = "复活",
    
        listener = {[ccs.TouchEventType.ended] = function() 
            audio.playEffect(R.SOUND_BUTTON)
            local num = EquipSystem:getPropNum(4000)
            if num > 0 then
                EquipSystem:setPropNum(4000, num - 1)
                self.hpStar:setRate(num-1,false)
                scheduler.unscheduleGlobal(self.handler)
                self.handler = nil

                self:performWithDelay(function()
                    DataCenter:dispatchEvent({name = "PLAYER_REBORN"})
                    display:resume();
                    self:removeSelf(true)    
                end, 0.5)   
            else
                local packageLayer = PackageLayer:new()
                packageLayer:setupReviveUI(function(result)
                       if result == PAY.PAY_STATE_SUCCESS then
                            self.hpStar:setRate( EquipSystem:getPropNum(4000),false)
                       end
                end) 
                self:addChild(packageLayer)
             end
         end},
    })

    self.resumeButton:setTitleFontSize(20)

    self.costMoney = ccs.label({
        text = "",
        x = display.cx,
        y = display.bottom + 50,
    })
    self.costMoney:setTouchEnabled(true)
    self.costMoney:setAnchorPoint(ccp(0.5,0.5))
    self.layer:addWidget(self.costMoney)

    self.exitButton = ccs.button({
        normal  = "ui/gameui/btn_exit_n.png",
        pressed = "ui/gameui/btn_exit_h.png",
        x = display.cx - 140,
        y = display.bottom + 80,
        listener = {[ccs.TouchEventType.began] = function() 
            audio.playEffect(R.SOUND_BUTTON)
            DataCenter:dispatchEvent({name = "GAME_FAILED"})
            self:removeSelf(true)
         end},
    })

    self.layer:addWidget(self.continueTitle)
    self.layer:addWidget(self.questionLabel)
    self.layer:addWidget(self.timeLabel)
    self.layer:addWidget(self.resumeButton)
    self.layer:addWidget(self.exitButton)

   -- if not LevelSystem._level.isForeverMode then
    self.taskInfoCell = TaskInfoCell.new()
    self.taskInfoCell:setup({ 
            max = 3,
            cur = 1,
            on = "ui/playerinfo/ui_stars_1.png",
            off = "ui/playerinfo/ui_stars_0.png",
            x = display.cx - 130,
            y = display.cy - 20,   
        })
   self:addChild(self.taskInfoCell)
   TaskSystem:checkAll()
   self.taskInfoCell:setTaskInfo(TaskSystem._tasks)
    -- end

    self.goldLabel = ui.newAtlasLabel({
        text = "0",
        start = '0',
        image = "ui/playerinfo/number_bar_gold.png",
        w = 16,
        h = 27,
        align = ui.TEXT_ALIGN_LEFT,
        x = display.right - 180,
        y = display.top - 60,
    })

    self.goldIcon = display.newSprite("ui/playerinfo/ui_gold.png", display.right - 215, display.top - 45)
    self:addChild(self.goldIcon)

    self:addChild(self.goldLabel)
    self.goldLabel:setString(GameSystem._record.numberOfGlod)

	return self
end

function ContinueLayer:updateTime( dt )

    self.time = self.time - 1  
    if self.time == 0 then
        DataCenter:dispatchEvent({name = "GAME_FAILED"})
        self:removeSelf(true)
    end

    if self.timeLabel then
        EffectHelper.playBubbleEffect(self.timeLabel, nil)
        self.timeLabel:setStringValue(self.time)
    end

end

function ContinueLayer:pauseTimer()
    if self.handler then
        scheduler.unscheduleGlobal(self.handler)
        self.handler = nil
    end
end

function ContinueLayer:resumeTime()
     self.handler = scheduler.scheduleGlobal(function(dt) self:updateTime(dt) end, 1.0)
end

function ContinueLayer:onEnter()
     self._eventHandler = function(event,args)
       self:eventHandler(event,args)
    end
    self.handler = scheduler.scheduleGlobal(function(dt) self:updateTime(dt) end, 1.0)
     DataCenter:addEventListener("GLOD_CHANGED_EVENT",self._eventHandler)
end

function ContinueLayer:onExit()
    if self.handler then
        scheduler.unscheduleGlobal(self.handler)
        self.handler = nil
    end

    DataCenter:removeEventListener("GLOD_CHANGED_EVENT",self._eventHandler)
end

function ContinueLayer:eventHandler(event)
    if event.name == "GLOD_CHANGED_EVENT" then
       self.goldLabel:setString(GameSystem._record.numberOfGlod)
    end
end

return ContinueLayer