--*******************************************
-- Desc: 开始场景
-- File: StartScene.lua
-- Author: zhouhaifeng
-- Date: 2014-03-20 10:07:33
--*******************************************

local AboutLayer = import("app.layers.AboutLayer")
local PackageLayer = import("app.layers.PackageLayer")

local StartScene = class("StartScene", function()
    return display.newScene("StartScene")
end)

function StartScene:ctor()
    self.isExpand = false
    self.bg = display.newSprite("ui/start/ui_start_bg.jpg", display.cx, display.cy)
    display.setIsBackgroundImage(self.bg)
    self:addChild(self.bg)

    -- self.uiTitle = display.newSprite("ui/start/ui_title.png", display.left + 210, display.top - 110)
    -- self:addChild(self.uiTitle)

    -- create menu
    self.startButton = ui.newImageMenuItem({
        image = "ui/start/ui_enter_0.png",
        imageSelected = "ui/start/ui_enter_0.png",
        x = display.cx,
        y = display.bottom + 155,
        listener = function()
            audio.playEffect(R.SOUND_BUTTON)
            MyApp:goToEquipment()
        end,
    })

    local action = CCSequence:createWithTwoActions(CCFadeIn:create(1.0), CCFadeOut:create(1.0))
    self.startButton:runAction(CCRepeatForever:create(action))

    self.settingButton = ui.newImageMenuItem({
        image = "ui/start/ui_setting_0.png",
        imageSelected = "ui/start/ui_setting_1.png",
        imageDisabled = "ui/start/ui_setting_1.png",
        x = display.left + 52,
        y = display.bottom + 52,
        listener = function()
            audio.playEffect(R.SOUND_BUTTON)
            self.isExpand = not self.isExpand
            self:setIsExpand()
        end,
    })

    self.soundButton = ui.newImageMenuItem({
        image = "ui/start/ui_sound_0.png",
        imageSelected = "ui/start/ui_sound_1.png",
        imageDisabled = "ui/start/ui_sound_1.png",
        x = display.left + 50,
        y = display.bottom + 130,
        listener = function()
            audio.playEffect(R.SOUND_BUTTON)
            local state = Setting:getSetting("VOLUME_MUSIC_ON")
            Setting:setSetting("VOLUME_MUSIC_ON",not state)
            self:fixMenuStates()
        end,
    })

    self.effectButton = ui.newImageMenuItem({
        image = "ui/start/ui_effect_0.png",
        imageSelected = "ui/start/ui_effect_1.png",
        imageDisabled = "ui/start/ui_effect_1.png",
        x = display.left + 50,
        y = display.bottom + 210,
        listener = function()
           audio.playEffect(R.SOUND_BUTTON)
           local state = Setting:getSetting("VOLUME_EFFECT_ON")
           Setting:setSetting("VOLUME_EFFECT_ON",not state) 
           self:fixMenuStates()
        end,
    })

    self.aboutButton = ui.newImageMenuItem({
        image = "ui/start/ui_aboutus_0.png",
        imageSelected = "ui/start/ui_aboutus_1.png",
        imageDisabled = "ui/start/ui_aboutus_1.png",
        x = display.left + 130,
        y = display.bottom + 52,
        listener = function()
           local aboutLayer = AboutLayer:new()
           self:addChild(aboutLayer)
        end,
    })


    self.recordButton = ui.newImageMenuItem({
        image = "ui/start/ui_record_0.png",
        imageSelected = "ui/start/ui_record_1.png",
        imageDisabled = "ui/start/ui_record_1.png",
        x = display.right - 135,
        y = display.bottom + 52,
        listener = function()
            audio.playEffect(R.SOUND_BUTTON)
            MyApp:goToRecord()
        end,
    })

    self.achievementButton = ui.newImageMenuItem({
        image = "ui/start/ui_achievement_0.png",
        imageSelected = "ui/start/ui_achievement_1.png",
        imageDisabled = "ui/start/ui_achievement_1.png",
        x = display.right - 55,
        y = display.bottom + 52,
        listener = function()
            audio.playEffect(R.SOUND_BUTTON)
            MyApp:goToAchievement()
        end,
    })

    self.supportButton = ui.newImageMenuItem({
      image = "ui/buy/huoli_1.png",
      imageSelected = "ui/buy/huoli_1.png",
      x = display.cx,
      y = display.bottom + 60,
      listener = function()
        audio.playEffect(R.SOUND_BUTTON)
         local packageLayer = PackageLayer:new()
         packageLayer:setupSupportUI(callback)
         self:addChild(packageLayer,100)
      end,
    })

    self.menu = ui.newMenu({
                            self.settingButton, self.startButton,
                            self.achievementButton,
                            self.effectButton,self.recordButton,
                            self.soundButton,self.aboutButton, self.supportButton 
                            })
    self:addChild(self.menu)
    
    self:setIsExpand()

    self:fixMenuStates()

    self:setTouchEnabled(true)
    self:setTouchMode(kCCTouchesAllAtOnce)
    self:addTouchEventListener(function(event,touches)
         MyApp:goToEquipment()
         self:setTouchEnabled(false) 
    end ,true, -127,false)
end

function StartScene:onEnter()
    SimpleAudioEngine:sharedEngine():setBackgroundMusicVolume(1)
    audio.playBackgroundMusic("sound/background.mp3",true);
     if device.platform == "android" then
        -- avoid unmeant back
        self:performWithDelay(function()
            -- keypad layer, for android
            local layer = display.newLayer()
            layer:addKeypadEventListener(function(event)
                if event == "back" then app.exit() end
            end)
            self:addChild(layer)

            layer:setKeypadEnabled(true)
        end, 0.5)
    end
end


--[[
    修正按钮状态
]]
function StartScene:fixMenuStates()

    if Setting:getSetting("VOLUME_EFFECT_ON") then
        self.effectButton:setNormalImage(CCSprite:create("ui/start/ui_effect_0.png"));
        self.effectButton:setSelectedImage(CCSprite:create("ui/start/ui_effect_1.png"))
        audio.isStatePlayEffect = true
    else
        audio.isStatePlayEffect = false
        self.effectButton:setNormalImage(CCSprite:create("ui/start/ui_effect_1.png"));
        self.effectButton:setSelectedImage(CCSprite:create("ui/start/ui_effect_0.png"))
    end
    
    if Setting:getSetting("VOLUME_MUSIC_ON") then
        self.soundButton:setNormalImage(CCSprite:create("ui/start/ui_sound_0.png"));
        self.soundButton:setSelectedImage(CCSprite:create("ui/start/ui_sound_1.png"))
        audio.isStatePlayMusic = true
        SimpleAudioEngine:sharedEngine():setBackgroundMusicVolume(1)
        audio.playMusic("sound/background.mp3",true);
    else
        audio.isStatePlayMusic = false
        audio.pauseMusic()
        self.soundButton:setNormalImage(CCSprite:create("ui/start/ui_sound_1.png"));
        self.soundButton:setSelectedImage(CCSprite:create("ui/start/ui_sound_0.png"))
    end

    Setting:saveSetting()
end


function StartScene:setIsExpand()
    self.effectButton:setVisible(self.isExpand)
    self.soundButton:setVisible(self.isExpand)
end

function StartScene:onExit()
  
end


return StartScene
