--*******************************************
-- Desc: 障碍物
-- File: Block.lua
-- Author: zhouhaifeng
-- Date: 2014-03-29 21:23:49
--*******************************************

local EffectManager = import("app.sprites.EffectManager")

local Block = class("Block",function(blockType)
	local prop = clone(BLOCK_PROPS[blockType])
	local node = display.newSprite(prop.image,0,0)
	node.params = prop
    return node
end)

--[[
	初始化
	params: 障碍物
]]
function Block:ctor(blockType)
	self.enable = true
    self:setBlockType(blockType)
end

--[[
	设置障碍物类型
]]
function Block:setBlockType(blockType)
	--如果是不同的类型的话
	if self.blockType ~= blockType then
		self:removeAllChildren()
		self.params = nil
		self.params = clone(BLOCK_PROPS[blockType])
		self.params.hp = 40
		self.params.isDestory = false
	end
	
	if blockType == 2 then
		local image = "maps/map" .. LevelSystem:getCurrentPlayMap() .."_msup_a.png"
		local pNewFrame = CCSpriteFrame:create(image,self.params.texRect)
		self:setDisplayFrame(pNewFrame)
		self.blockType = blockType
	else
		local pNewFrame = CCSpriteFrame:create("objects/cars/car_" .. math.random(1,8) ..".png",self.params.texRect)
		self:setDisplayFrame(pNewFrame)
		self.blockType = blockType
	end
	
	--for debug
    if DEBUG_LABEL then
        self.nameLabel = ui.newTTFLabel({
            x = 0,
            y = 0,
            width = 80,
            height = 20,
            text = "0"
        })
        self:addChild(self.nameLabel)

    end

 --    local rect = display.newRect(self:getEventRect())
 --    rect:setAnchorPoint(ccp(0,0))
 --    rect:setPosition(0,0)
	-- self:addChild(rect)

    self:getTexture():setAliasTexParameters()
	self:setAnchorPoint(ccp(0,0))
   
end

function Block:onHit(damage)
	if self.params.isDestory then
		return 
	end
	
	local action1 = CCTintTo:create(0.1,220, 0, 0)
    local action2 = CCTintTo:create(0.1,255, 255, 255)
    self:runAction(CCSequence:createWithTwoActions(action1,action2))
    EffectHelper.hitEffect(self,damage,nil)

    self.params.hp = self.params.hp - damage
	if self.params.hp < 0 then
		self.params.isDestory = true
	end
	
end

--[[
	被摧毁
]]
function Block:onDestoryed()
	
end


--[[
	获取碰撞区域
]]
function Block:getCollisionRect()
	return CCRectMake(
				self:getPositionX()+16,
				self:getPositionY()+16,
			    self.params.hitrect.width * 32 - 16,
			    self.params.hitrect.height * 32 - 16
			   )
end

--[[
	获取碰撞区域
]]
function Block:getEventRect()
	return CCRectMake(
				self:getPositionX() + self.params.eventRect.origin.x,
				self:getPositionY() + self.params.eventRect.origin.y,
			    self.params.eventRect.size.width,
			    self.params.eventRect.size.height
			   )
end

--[[
	爆炸
]]

function Block:onExplode()
	DataCenter:dispatchEvent({name = "CAR_DISTORYED"})
	EffectManager:playExplodeEffect(self)
	audio.playEffect(R.SOUND_BIG_BOMB,false)
	local pNewFrame = CCSpriteFrame:create("objects/cars/car_0.png",self.params.texRect)
	self:setDisplayFrame(pNewFrame)
end



return Block