--*********************************************
-- Desc: 子弹类库
-- File: Bullet.lua
-- Author: zhouhaifeng
-- Date: 2014-04-03 14:25:37
--*********************************************

local EffectManager = import("app.sprites.EffectManager")

local Bullet = class("Bullet",function(bulletType)
	local prop = BULLET_PROPS[bulletType]
	local node = display.newSprite(prop.image,0,0)
	node.params = prop
    return node
end)

--[[
	初始化
	params: 子弹类型
]]
function Bullet:ctor(bulletType)
	self.enabled = true
	self.rotation = 0
    self:setBulletType(bulletType)
end


--[[
	设置子弹类型
]]
function Bullet:setBulletType(bulletType)
	--如果是不同的类型的话
	self:stopAllActions();
	if self.bulletType ~= bulletType then
		self.params = nil
		self.params = BULLET_PROPS[bulletType]
		local pNewFrame = CCSpriteFrame:create(self.params.image,self.params.texRect)
		self:setDisplayFrame(pNewFrame)
		self.bulletType = bulletType
	end
	self.enabled = true 
	self:getTexture():setAntiAliasTexParameters()
	self:setAnchorPoint(ccp(0,0))
end

--[[
	设置射击角度
]]
function Bullet:setDirectRotation(angle)
	self.rotation = angle
	self:setRotation(angle)	
end

--[[
	获取子弹的伤害范围
]]
function Bullet:getCollisionRect()
	--榴弹枪子弹
	if self.bulletType == kWeaponTypeRPG then
		return CCRectMake(300 ,0 ,400 ,display.height)
	--散弹枪子弹
	elseif self.bulletType == kWeaponTypeShotGun then
		local x,y = self:getPosition()
		local size = self:getContentSize()
		return CCRectMake(x,y,size.width,size.height)
	--普通枪子弹
	else
		local x,y = self:getPosition()
		local size = self:getContentSize()
		return CCRectMake(x,y,size.width,size.height)
	end
end

--[[
	发射
]]
function Bullet:onFire()
	local mapLayer = Global:get("mapLayer")  
    self:removeFromParentAndCleanup(false)
    mapLayer.tileMap:addChild(self,400)

    if self.bulletType ~= kWeaponTypeRPG then
	    local objectsManager = Global:get("objectsManager")
		objectsManager:addBulletCollision(self)   
	end

	if self.bulletType == kWeaponTypeRifle or self.bulletType == kWeaponTypeShotGun then
		local machineGunEffect = EffectManager:playMachineGunEffect(mapLayer.tileMap)
  		machineGunEffect:setPosition(self:getPosition())    
	elseif self.bulletType == kWeaponTypeHandGun then
		local handGunEffect = EffectManager:playHandGunEffect(mapLayer.tileMap)
  		handGunEffect:setPosition(self:getPosition())   
	end
  
	self:playOnFireEffect()
	self:playBulletAnimtion()
end

--[[
	命中一只怪物
]]
function Bullet:onHitOne(index)
	if self.bulletType ~= kWeaponTypeRPG then
		local objectsManager = Global:get("objectsManager")
		objectsManager:removeBulletCollision(index)
		self:onDestory()
	end		
end

--[[
	播放开火音效
]]
function Bullet:playOnFireEffect()
	if self.bulletType == kWeaponTypeRPG then
		audio.playEffect(R.SOUND_RPG_FIRE,false)
    elseif self.bulletType == kWeaponTypeShotGun then
    	audio.playEffect(R.SOUND_SHOOT_GUN,false)
	else
		audio.playEffect(R.SOUND_HAND_GUN,false)
	end
end

--[[
	播放开火音效
]]
function Bullet:playOnExplodeEffect()
	if self.bulletType == kWeaponTypeRPG then
		audio.playEffect(R.SOUND_RPG_EXPLODE,false)
	end
end

--[[
	播放开火动画
]]
function Bullet:playBulletAnimtion()
	
	if self.bulletType == kWeaponTypeRPG then
		self:setDirectRotation(0)
		self:setVisible(false)
		local toX = 0 * math.cos(self.rotation) 
		local toY = 0 * math.sin(self.rotation)  
		transition.moveBy(self, {time = 0, x = toX, y = toY ,onComplete = function()
			self:onDestory()
		end})
	elseif self.blockType == kWeaponTypeShotGun then
		self:setDirectRotation(0)
		local toX = 800 * math.cos(self.rotation) 
		local toY = 0 * math.sin(self.rotation)  
		transition.moveBy(self, {time = 0.8 , x = toX, y = toY ,onComplete = function()
			self:onDestory()
		end})
	else
		self:setDirectRotation(0)
		local toX = 800 * math.cos(self.rotation) 
		local toY = 0 * math.sin(self.rotation)  
		transition.moveBy(self, {time = 0.5 , x = toX, y = toY ,onComplete = function()
			self:onDestory()
		end})
	end
end

--[[
	爆炸
]]
function Bullet:onDestory()
	if self.bulletType == kWeaponTypeRPG then
		local objectsManager = Global:get("objectsManager")
		objectsManager:addEffect(EffectManager:playRPGBombEffect(self:getParent(),ccp(450,240),0.8))
		objectsManager:addEffect(EffectManager:playRPGBombEffect(self:getParent(),ccp(550,150),1.1))
		objectsManager:addEffect(EffectManager:playRPGBombEffect(self:getParent(),ccp(445,100),0.85))
		objectsManager:killWithBomb(self:getCollisionRect())
		self:playOnExplodeEffect()
	else
		 local spark = EffectManager:playSparkEffect(self:getParent())
		 if spark ~= nil then
		 	 spark:setPosition(self:getPosition())
		 end
	end

	self.enabled = false

	local BulletPools = Global:get("BulletPools")  
	self:removeFromParentAndCleanup(false)
	BulletPools:releaseObjects(self)
end    

return Bullet