--*********************************************
-- Desc: 效果
-- File: EffectHelper.lua
-- Author: zhouhaifeng
-- Date: 2014-03-14 11:34:47
--*********************************************

local EffectHelper = {}

local EFFECT_COLORS = {
    ["info"]    = ccc3(255,255,255),       
    ["warn"]    = ccc3(192,24,24),
    ["bonus"]   = ccc3(192,192,24),
    ["green"]   = ccc3(24,192,24),
}


--[[
    按钮的果冻弹跳效果
]]
function EffectHelper.playBubbleEffect( node, callback )

	node:setScaleX(1.0)
	node:setScaleY(1.0)
	local function zoom1(node ,offset, time, onComplete)
      		
            local size = node:getContentSize()
       
            local scaleX = node:getScaleX() * (size.width + offset) / size.width
            local scaleY = node:getScaleY() * (size.height - offset) / size.height
         
            transition.scaleTo(node, {
                scaleX     = scaleX,
                scaleY     = scaleY,
                time       = time,
                onComplete = onComplete,
            })
        end

    local function zoom2(node,offset, time, onComplete)
        local size = node:getContentSize()
        transition.scaleTo(node, {
            scaleX     = 1.0,
            scaleY     = 1.0,
            time       = time,
            onComplete = onComplete,
        })
    end

    zoom1(node,10, 0.08, function()
        zoom2(node,10, 0.09, function()
            zoom1(node,5, 0.10, function()
                zoom2(node,5, 0.11, function()
                    if callback then
                         callback(tag)
                    end
                end)
            end)
        end)
    end)
end


--[[
    快速的弹跳移入
]]
function EffectHelper.BounceMoveInEffect(node, params, callback )
    local action = CCMoveTo:create(params.time, ccp(params.x,params.y))
    local newAction = transition.newEasing(action, "BACKINOUT")
    node:runAction(newAction)
end

function EffectHelper.WidgetBobbleInEffect(uiwidget,onComplete)
    uiwidget:setScale(0.5)
    local scale = CCScaleTo:create(0.35,1.0)
    local callfun = CCCallFunc:create(function()
        if onComplete then
            onComplete()
        end
    end)

    local bounce = CCEaseBackOut:create(scale)
    local action = CCSequence:createWithTwoActions(bounce,callfun)
    uiwidget:runAction(action)
end

--[[
   果冻弹出的效果
]]
function EffectHelper.WidgetBobbleOutEffect(uiwidget,onComplete)
    uiwidget:setScale(1.0)
    local scale = CCScaleTo:create(0.2,0.6)
    local alpha = CCFadeTo:create(0.2,0.6)
    local spawn = CCSpawn:createWithTwoActions(alpha, scale)
    local callfun = CCCallFunc:create(function()
        if onComplete then
            onComplete()
        end
    end)

    local bounce = CCEaseBounceOut:create(spawn)
    local action = CCSequence:createWithTwoActions(spawn,callfun)
    uiwidget:runAction(action)
end

--[[
    滚动增加的动画效果
]]
function EffectHelper.RollNumEffect(uiwidget,time,fromValue,toValue,prefix,suffix) 
    local _prefix = prefix or ""
    local _suffix = suffix or ""
    local _type =  tolua.type(uiwidget)

    local function setValue(uiwiget,value) 
        if _type == "UILabel" then
            uiwidget:setText(_prefix .. math.ceil(value) .. _suffix)
        elseif _type == "UILabelAtlas" then
            uiwidget:setStringValue(_prefix .. math.ceil(value) .. _suffix)
        elseif _type == "UILoadingBar" then
            uiwidget:setPercent(tonumber(value))
        end
    end

    local function play() 
        local times = 30.0
        local interval = time / times   
        local interval_num = (toValue-fromValue)/ times
        local handle 
        local cur_num = fromValue
        local cur_times = times 
 
        handle = CCDirector:sharedDirector():getScheduler():scheduleScriptFunc(function()
            xpcall(function() 
                    if cur_times <= 0 then 
                        CCDirector:sharedDirector():getScheduler():unscheduleScriptEntry(handle)
                        handle = nil 
                        --set num
                        setValue( uiwidget , toValue)
                        return 
                    end 
                    --set num
                    setValue(uiwidget, cur_num)
                    cur_num = cur_num + interval_num
                
                    -- if cur_num > toValue then 
                    --     cur_num = toValue 
                    -- end 
                    cur_times = cur_times - 1 
                end , function() CCDirector:sharedDirector():getScheduler():unscheduleScriptEntry(handle) handle = nil end) 
        end , interval , false) 
    end 

    play() 
end 

--[[
    打字机效果
]]
function EffectHelper.playTyperEffect(node,toValue,callback,interval)
    local times = string.len(toValue) 
    local interval = interval or 0.04
    local handle 
    local curTimes = 0 
    
    handle = CCDirector:sharedDirector():getScheduler():scheduleScriptFunc(function()
        xpcall(function() 
                if curTimes >= times then 
                    CCDirector:sharedDirector():getScheduler():unscheduleScriptEntry(handle)
                    handle = nil 
                    --set num
                    node:setString(toValue)

                    if callback then
                        callback()
                    end
                    return 
                end 
                --set num
                curTimes = curTimes + 1 
                node:setString(string.sub(toValue,0,curTimes))

            end , function() CCDirector:sharedDirector():getScheduler():unscheduleScriptEntry(handle) handle = nil end) 
    end , interval , false) 
end



--[[
    浮动动画
]]
local function createAction(target , time , args)
        target:setVisible( true )
        local scene = display.getRunningScene()
        if not args then args = {} end
        local showTime = time  -- 动画效果总时间
        local startEfTime = showTime / 5  -- 显示前渐变动画效果时间
        local endEfTime = showTime / 5  -- 消失时渐变动画时间
        local move_y = args.y or 100

        target.isAction = true
    
        scene:addChild(target,1000)

        startEfTime = 0.2
        endEfTime = 0.2

        -- 渐变显示
        transition.fadeIn(target , {
            time = startEfTime,
        })

        transition.moveBy(target , {
            y = 20,
            time = startEfTime,
            onComplete = function()
                transition.moveBy(target , {
                    delay = showTime,
                    y = 20,
                    time = endEfTime,
                    onComplete = function()
                        target:setVisible(false)
                        target:removeFromParentAndCleanup(true) -- 清除自己
                        target.isAction = false
                    end
                })
            end
        })

        -- 消失动画
        transition.fadeOut(target , {
            time = endEfTime,
            delay = showTime + startEfTime,
        })
end

--[[
    浮动的消息提示
]]
function EffectHelper.floatMessage(str,type,pos) 
        --单纯文字展示
        local function textShow(str,duration)
            local time = duration
            if time == 0 then return end

            local sprite = display.newSprite("ui/common/ui_message_bg.png", 0,0)
            local textField = CCLabelTTF:create(str , "Arial" , 18)
            textField:setAnchorPoint( ccp(0.5 , 0.5) )
            textField:setColor(ccc3(255,255,255))
            textField:setPosition(sprite:getContentSize().width/2, 25)
            sprite:addChild(textField)

            local tempX = display.cx
            local tempY = display.bottom + 100
            sprite:setPosition( ccp(tempX , tempY) )
            createAction(sprite , time , {y=100})

            return sprite
        end

        return textShow(str,2)
end

--[[
    浮动的消息提示
]]
function EffectHelper.floatTipInfo(str,type,pos) 

        --单纯文字展示
        local function textShow(str,duration,pos)
            local time = duration
            if time == 0 then return end

         
            local textField = CCLabelTTF:create(str , "Arial" , 18)
            local sprite = display.newScale9Sprite("ui/common/ui_common_tips_bg.png", 0,0,CCSizeMake(textField:getContentSize().width + 40, textField:getContentSize().height + 40))

            textField:setAnchorPoint( ccp(0.5 , 0.5) )
            textField:setColor(ccc3(255,255,255))
            textField:setPosition(sprite:getContentSize().width/2, sprite:getContentSize().height/2)
            sprite:addChild(textField)

            if pos then 
                sprite:setPosition(pos)
                createAction(sprite , time , {y=100})
            else 
                local tempX = display.cx
                local tempY = display.cy
                sprite:setPosition( ccp(tempX , tempY) )
                createAction(sprite , time , {y=100})
            end

            return sprite
        end

        return textShow(str,1,pos)
end

local spriteList = {}
--[[
    浮动的消息提示
]]
function EffectHelper.reset()
    spriteList = {}
end 

function EffectHelper:cleanupfloatAimInfo()
    for k,v in pairs(spriteList) do
        v:removeFromParentAndCleanup(true)
    end
    spriteList = {}
end

function EffectHelper.floatAimInfo(str,type,pos) 
        
        local function doFloatAction(target , time , args)
                target:setVisible( true )
                local scene = display.getRunningScene()
                if not args then args = {} end
                local showTime = time  -- 动画效果总时间
                local startEfTime = showTime / 5  -- 显示前渐变动画效果时间
                local endEfTime = showTime / 5  -- 消失时渐变动画时间
                local move_y = args.y or 100

                target.isAction = true
            
                scene:addChild(target)

                startEfTime = 0.2
                endEfTime = 0.2



                -- 渐变显示
                transition.fadeIn(target , {
                    time = startEfTime,
                })

                transition.moveBy(target , {
                    y = 20,
                    time = startEfTime,
                    onComplete = function()
                        transition.moveBy(target , {
                            delay = showTime,
                            y = 20,
                            time = endEfTime,
                            onComplete = function()
                                target:setVisible(false)
                                target:removeFromParentAndCleanup(true) -- 清除自己
                                target.isAction = false
                                table.removeItem(spriteList, target)
                            end
                        })
                    end
                })

                -- 消失动画
                transition.fadeOut(target , {
                    time = endEfTime,
                    delay = showTime + startEfTime,
                })
        end

        --单纯文字展示
        local function textShow(str,duration,pos)
            local time = duration
            if time == 0 then return end

         
            local textField = CCLabelTTF:create(str , "Arial" , 18)
            local sprite = display.newScale9Sprite("ui/common/ui_common_aim_bg.png", 0,0,CCSizeMake(textField:getContentSize().width + 40, textField:getContentSize().height + 40))

            textField:setAnchorPoint( ccp(0.5 , 0.5) )
            textField:setColor(ccc3(255,255,255))
            textField:setPosition(sprite:getContentSize().width/2, sprite:getContentSize().height/2)
            sprite:addChild(textField)

            if pos then 
                sprite:setPosition(pos)
                doFloatAction(sprite , time , {y=100})
            else 
                local tempX = display.right - 150
                local tempY = display.bottom + 120
                sprite:setPosition( ccp(tempX , tempY) )
                doFloatAction(sprite , time , {y=100})
            end

            return sprite
        end

        local sprite = textShow(str,1.5,pos)

        for k,v in pairs(spriteList) do
                v:runAction(CCMoveBy:create(0.2, ccp(0,60)))
        end

         table.insert(spriteList, sprite)

end

--[[
    浮动的消息提示
]]
function EffectHelper.hitEffect(parent,num,param) 

    if type(param) ~= "table" then param = {} end
    
    if tonumber( num ) == 0 then
        if param.onComplete then param.onComplete() end
        return
    end
    
    local node
    if param.isCriticalStrikes then
        --暴击掉血
        node = ui.newAtlasLabel({
            text = num,
            start = '0',
            image = "ui/common/ui_harm_num.png",
            w = 18,
            h = 18,
            x = parent:getPositionX() + parent:getContentSize().width/2 + math.random(-20,20) ,
            y = parent:getPositionY() + parent:getContentSize().height + math.random(0,20) ,
        })
    else
        --掉血或者加血
         node =  ui.newAtlasLabel({
            text = num,
            start = '0',
            image = "ui/common/ui_harm_num.png",
            w = 18,
            h = 18,
            x = parent:getPositionX() + parent:getContentSize().width/4 + math.random(-10,10) ,
            y = parent:getPositionY() + parent:getContentSize().height + math.random(0,20) ,
        })
    end

        --[[特效开始]]
    node:setScale(0.5)
    
    parent:getParent():addChild(node,200)
    node:setAnchorPoint(ccp(0.5,0.5))


    transition.scaleTo(node, {
        time = 0.1,
        scale = 0.8 + math.random(0.2,0.3),
    })
    transition.scaleTo(node, {
        delay = 0.2,
        time = 0.15,
        scale = 1,
    })
    
    transition.fadeOut(node, {
        delay = 0.3,
        time = 0.2,
        onComplete = function()
            node:removeFromParentAndCleanup(true)  -- 清除自己
            if param.onComplete then param.onComplete() end
        end
    })
    
end

function EffectHelper:shakeScreenEffect(node)
    local pos = CCPoint(0, 0)
    local callback = CCCallFunc:create(function()
        node:setPosition(pos)
    end)

    local move1 = CCMoveBy:create(0.04,ccp(4,4));
    local move2 = CCMoveBy:create(0.04,ccp(-8,-8));
    local move3= CCMoveBy:create(0.02,ccp(6,6));
    local move4 = CCMoveBy:create(0.02,ccp(-4,-4));
    local move5 = CCMoveBy:create(0.01,ccp(2,2));

    local array = CCArray:create()
    array:addObject(move1)
    array:addObject(move2)
    array:addObject(move3)
    array:addObject(move4)
    array:addObject(move5)
    array:addObject(callback)
    local action = CCSequence:create(array);

    node:runAction(action);
end

return EffectHelper

 