--*********************************************
-- Desc: 特效控制类
-- File: EffectManager.lua
-- Author: zhouhaifeng
-- Date: 2014-04-16 18:22:47
--*********************************************

local EffectManager = {}

function EffectManager:init()
	display.addSpriteFramesWithFile("animations/effects/bomb.plist", "animations/effects/bomb.pvr.ccz")
	local frames = display.newFrames("bomb_%d.png", 0, 10)
    local animation = display.newAnimation(frames, 0.06)
    display.setAnimationCache("bombEffect", animation)

    display.addSpriteFramesWithFile("animations/effects/hit.plist", "animations/effects/hit.pvr.ccz")
	local frames = display.newFrames("hitEffect_%d.png", 1, 6)
    local animation = display.newAnimation(frames, 0.06)
    display.setAnimationCache("sparkEffect1", animation)

	local frames = display.newFrames("bloodEffect_%d.png", 1, 5)
    local animation = display.newAnimation(frames, 0.06)
    display.setAnimationCache("sparkEffect2", animation)

    local frames = display.newFrames("bloodEffectBig_%d.png", 1, 4)
    local animation = display.newAnimation(frames, 0.06)
    display.setAnimationCache("sparkEffect3", animation)

    display.addSpriteFramesWithFile("animations/effects/exhaust.plist", "animations/effects/exhaust.png")
	local frames = display.newFrames("exhaust_%d.png", 0, 4)
    local animation = display.newAnimation(frames, 0.1)
    display.setAnimationCache("exhaustEffect", animation)

    display.addSpriteFramesWithFile("animations/effects/soot.plist", "animations/effects/soot.png")
	local frames = display.newFrames("soot_%d.png", 0, 5)
    local animation = display.newAnimation(frames, 0.15)
    display.setAnimationCache("sootEffect", animation)

    display.addSpriteFramesWithFile("animations/effects/fire.plist", "animations/effects/fire.pvr.ccz")
	local frames = display.newFrames("handGun_%d.png", 1, 3)
    local animation = display.newAnimation(frames, 0.08)
    display.setAnimationCache("handGunEffect", animation)

	local frames = display.newFrames("machineGun_%d.png", 1, 4)
    local animation = display.newAnimation(frames, 0.06)
    display.setAnimationCache("machineGunEffect", animation)

    local frames = display.newFrames("rpgGun_%d.png", 1, 4)
    local animation = display.newAnimation(frames, 0.1)
    display.setAnimationCache("rpgGunEffect", animation)
end

--[[
	汽车爆炸特效
]]
function EffectManager:playExplodeEffect(node)

	local explode = display.newSprite("#bomb_0.png")

	local callback = CCCallFunc:create(function()
		explode:removeFromParentAndCleanup(true)
	end)

	local array = CCArray:create()
	array:addObject(CCAnimate:create(display.getAnimationCache("bombEffect")))
	array:addObject(callback)
	local action = CCSequence:create(array)
    node:addChild(explode)

    explode:runAction(action)
    explode:setAnchorPoint(ccp(0.5,0))
    explode:setPosition(node:getContentSize().width/2, 0)

    EffectHelper:shakeScreenEffect(display:getRunningScene())
end

--[[
	榴弹爆炸效果
]]
function EffectManager:playRPGBombEffect(layer,pos,scale)

	local explode = display.newSprite("#bomb_0.png")

	local callback = CCCallFunc:create(function()
		-- explode:removeFromParentAndCleanup(true)
	end)

	local array = CCArray:create()
	array:addObject(CCAnimate:create(display.getAnimationCache("bombEffect")))
	array:addObject(callback)
	local action = CCSequence:create(array)
    layer:addChild(explode,1000)

    explode:setScale(scale)
    explode:runAction(action)
    explode:setAnchorPoint(ccp(0.5,0))
    explode:setPosition(pos)

	EffectHelper:shakeScreenEffect(display:getRunningScene())
    return explode
end


--[[
	怪物被击中的特效
]]
function EffectManager:playSparkEffect(node)
	if node == nil then
		return 
	end

	local sparkType = math.random(1,3)
	local sparkFileName = ""
	if sparkType == 1 then
		sparkFileName = "#hitEffect_1.png"
	elseif sparkType == 2 then
		sparkFileName = "#bloodEffect_1.png"
	else 
		sparkFileName = "#bloodEffectBig_1.png"
	end

	local spark = display.newSprite(sparkFileName)
	if spark == nil then
		return 
	end

	local callback = CCCallFunc:create(function()
		spark:removeFromParentAndCleanup(true)
	end)

	local array = CCArray:create()
	array:addObject(CCAnimate:create(display.getAnimationCache("sparkEffect"..sparkType)))
	array:addObject(callback)
	local action = CCSequence:create(array)
    node:addChild(spark,1000)
    spark:setScale(3);
    spark:runAction(action)
 
    return spark
end

--[[
	手枪的特效
]]
function EffectManager:playHandGunEffect(node)
	
	local effect = display.newSprite("#handGun_1.png")

	local callback = CCCallFunc:create(function()
		effect:removeFromParentAndCleanup(true)
		effect = nil
	end)

	local array = CCArray:create()
	array:addObject(CCAnimate:create(display.getAnimationCache("handGunEffect")))
	array:addObject(callback)
	local action = CCSequence:create(array)

	effect:setScaleX(-1)
    node:addChild(effect,2000)
    effect:setAnchorPoint(ccp(0,0.4))
    effect:runAction(action)
    return effect
end

--[[
	机关枪的特效
]]
function EffectManager:playMachineGunEffect(node)
	
	local effect = display.newSprite("#machineGun_1.png")

	local callback = CCCallFunc:create(function()
		effect:removeFromParentAndCleanup(true)
		effect = nil
	end)

	local array = CCArray:create()
	array:addObject(CCAnimate:create(display.getAnimationCache("machineGunEffect")))
	array:addObject(callback)
	local action = CCSequence:create(array)

    node:addChild(effect,2000)
    effect:setScale(-0.5)
    effect:setAnchorPoint(ccp(0,0.6))
    effect:runAction(action)
    return effect
end

--[[
	Rpg的特效
]]
function EffectManager:playRpgGunEffect(node)
	
	local effect = display.newSprite("#rpgGun_1.png")

	local callback = CCCallFunc:create(function()
		effect:removeFromParentAndCleanup(true)
		effect = nil
	end)

	local array = CCArray:create()
	array:addObject(CCAnimate:create(display.getAnimationCache("rpgGunEffect")))
	array:addObject(callback)
	local action = CCSequence:create(array)

    node:addChild(effect,2000)
    effect:setAnchorPoint(ccp(0,0.4))
    effect:runAction(action)
    return effect
end

--[[
	地上的泥土特效
]]
function EffectManager:playSootEffect(node)
	
	local soot = display.newSprite("#soot_0.png")

    node:addChild(soot,1000)
    soot:setPosition(ccp(-90,18))
    soot:runAction(CCRepeatForever:create(CCAnimate:create(display.getAnimationCache("sootEffect"))))
    return soot
end


--[[
	摩托车烟尘特效
]]
function EffectManager:playExhaustEffect(node)
	local exhaust = display.newSprite("#exhaust_0.png")
    node:addChild(exhaust,1001)
    exhaust:runAction(CCRepeatForever:create(CCAnimate:create(display.getAnimationCache("exhaustEffect"))))
    return exhaust
end



return EffectManager