--*********************************************
-- Desc: 僵尸怪物
-- File: Monster.lua
-- Author: zhouhaifeng
-- Date: 2014-03-17 16:24:54
--*********************************************

local MonsterItems = import("app.datas.MonsterItems")
local EffectManager = import("app.sprites.EffectManager")
local Monster = class("Monster",function()
    --获取新的状态
    return CCNodeExtend.extend(CCArmature:create()) 
end)
--[[
	初始化
	params: 角色信息
]]
function Monster:ctor(name)
    self:setNodeEventEnabled(true)
    self:loadConfig(name);
    self:reset()
    if DEBUG_LABEL then
        local width = self:boundingBox().size.width
        local height = self:boundingBox().size.height
        local rect = display.newRect(width,height)
        rect:setAnchorPoint(ccp(0,0))
        rect:setPosition(self:boundingBox().origin.x,height/2+self:boundingBox().origin.y)
        self:addChild(rect)
    end
end

--[[
    重置怪物状态
]]
function Monster:reset()
    self:initFSM();

    local monster = MonsterItems[self.monsterType]

    self.config = {}
    self.config.isAttack = false
    self.config.isDead = false
    self.config.isBurn = false          -- 是否着火
    self.config.isBoss = false          -- 是否着火
    self.config.isStuck = false        --  卡住    
    self.config.sensitivity = 1.0 / 5        -- 横向速度
    self.config.velocity = 1.0  / 5          -- 加速度
    self.config.maxHp = monster.hp      -- 最大血量
    self.config.maxAtk = monster.atk    -- 最大攻击
    self.config.hp = self.config.maxHp  -- 最大血量
    self.config.maxSpeed = 5
    self.config.currentSpeed = self.config.maxSpeed

    self.params = {}    
    self.params.hitrect = CCSize(1,1)
    self.params.eventRect = CCRectMake(0,0,12,12)
    self.params.enabled = true
    self.params.pos = ccp(- math.random(1,2),math.random(2,6))
    self.params.enabled = true
    self.params.isBurn = false

    if self.monsterType == 1012 or self.monsterType == 1013 or self.monsterType == 1014 then
         self.config.isBoss = true
    else
         self.config.isBoss = false
    end
   
    self.config.currentActionName = "stop"
    self.config.collisionEnabled = true

    self:setAnchorPoint(ccp(0.5,0))
    self:stopAllActions()
    self:setScaleX(1)

    self:setOpacity(255)
    self:runAction(CCTintTo:create(0.0,255, 255, 255))

end

function Monster:onEnter()
    self.handle = scheduler.scheduleGlobal(function()
        self:updateLogic()
    end,0.1)
end

function Monster:onExit()
    if self.handle then
        scheduler.unscheduleGlobal(self.handle)
        self.handle = nil
    end
end


function Monster:loadConfig(name)
    self.monsterType = 0
    local propId = tonumber(name)
    self:setMonsterType(propId)

    self.animationNames = {
        standby = "daiji",
        walk = "zoulu",
        run = "paobu",
        attack = "gongji",
        runfire = "paobu_fire",
        dead = "dasi",
        hitdead = "zhuangsi_1"
    }
end

--[[
    设置怪物类型
]]
function Monster:setMonsterType(propId)

    if self.monsterType == propId then
        return 
    end

    if self.animation then
        self.animation:removeScriptMovementEventHandler()
        self:removeFromParent()
    end

    local prop = MonsterItems[propId];    
    local manager = CCArmatureDataManager:sharedArmatureDataManager()

    if propId == 1012 or propId == 1013 or propId == 1014 then
        local bossId = propId - 1011
        local animationFileName = "animations/boss_" .. bossId
        manager:addArmatureFileInfo(animationFileName ..".ExportJson")
        self:init("boss_" .. bossId)
    else
        local animationFileName = "animations/monster_" .. propId
        manager:addArmatureFileInfo(animationFileName ..".ExportJson")
        self:init("monster_" .. propId)
    end
   

    self.animation = self:getAnimation()
    self.animation:setSpeedScale(1)
    self.movements = {}
    self.animation:setScriptMovementEventHandler(function(movementId, state)
        local complete = self.movements[movementId]
        if complete then
            complete(state)
        end
    end)

     self.monsterType = propId
end


--[[
	初始化状态机
]]
function Monster:initFSM()
	self.fsm_ = {}
    cc.GameObject.extend(self.fsm_)
        :addComponent("components.behavior.StateMachine")
        :exportMethods()

    self.fsm_:setupState({
        events = {
            --待机
            {name = "tostandby", from = {"none","walk","run","attack","pursuit"}, to = "standby" },
        	--走动
            {name = "towalk", from = {"none","standby","run","attack","pursuit"} , to = "walk"},
            --跑步
            {name = "torun",  from = {"none","standby","walk","attack","pursuit"} , to = "run"},
            --攻击
            {name = "toattack",from = {"none","standby","walk","attack","pursuit"}, to = "attack"},
            --受到攻击
            {name = "attacked"},
            --追击主角
            {name = "topursuit", from = {"none","standby","run","walk","attack"}, to = "pursuit"},
            --燃烧动画
            {name = "toburn"},
            --死亡
            {name = "todead" , to = "dead"},
            --死亡
            {name = "tohitdead" , to = "dead"},
        },
        callbacks = {
            ontostandby  = function(event)
                            self:stopAllActions()
            			 -- CCLuaLog("[FSM] ontostandby" .. self.config.currentSpeed) 
                            self:playAction("standby")
                        end,
            ontowalk    = function (event)
                            if self.config.isStuck then 
                                return 
                            end

                            self.config.currentSpeed = 5
                            self:playAction("walk")

                            local scale = math.random(-1,1)    
                            local array = CCArray:create()
                            
                            if scale < 0 then
                                scale = -1
                            else 
                                scale = 1
                            end

                            array:addObject(CCScaleTo:create(0.0,  scale,1))
                            array:addObject(CCMoveBy:create(2.0, ccp( scale * 50,0)))
                            array:addObject(CCScaleTo:create(0.0,  -scale,1)) 
                            array:addObject(CCMoveBy:create(2.0, ccp( -scale * 50,0)))
                                             
                            self:runAction(
                                CCRepeat:create(
                                        CCSequence:create(
                                                array
                                            )
                                        ,3)
                                    )

                        end,

            ontorun     = function(event) 
	                   		    self:playAction("run")
                               
                        end,
 
            ontoattack  = function(event) 

                            self:stopAllActions()

                            local player = Global:get("player") 
                            local playerX,playerY = player.avstar:getPosition()
                            local monsterX,monsterY = self:getPosition()
                            local distance = ccpDistance(ccp(playerX,playerY), ccp(monsterX,monsterY))
                            
                            if playerX > monsterX then
                                self:setScaleX(1)
                             else 
                                self:setScaleX(-1)
                            end

                            self:runAction(CCMoveTo:create(distance / 100, ccp(playerX + 50, playerY)))

                            local player = Global:get("player")
                            --玩家没死的话then
                            if not player.config.isDead then
                                self.config.isAttack = true 
                                self:playAction("attack",function(state)
                                    if state == 1 then
                                        self.config.isAttack = false 

                                        local player = Global:get("player")

                                        local rect = player.avstar:boundingBox()
                                        local contentRect = self:boundingBox()
                                        if  contentRect:intersectsRect(rect) then
                                            local damage = self:getAttackDamage()
                                            player:doEvent("attacked",damage)
                                        end
                                    end
                                end)	
                            else
                                self:doEvent("tostandby")
                            end
                        end,
            onpursuit = function(event)
                    
                            if self.config.isDead then
                                return
                            end

                             if self.config.isStuck then 
                                return 
                            end

                            self:stopAllActions()

                            local player = Global:get("player") 
                            local playerX,playerY = player.avstar:getPosition()
                            local monsterX,monsterY = self:getPosition()
                            local distance = ccpDistance(ccp(playerX,playerY), ccp(monsterX,monsterY))
                            
                            if playerX > monsterX then
                                self:setScaleX(1)
                             else 
                                self:setScaleX(-1)
                            end

                            self:playAction("walk")
                            self:runAction(CCMoveTo:create(distance / 100, ccp(playerX + math.random(-1,1) * 20, playerY + math.random(-1,1) * 20)))
                            
                        end,
            onattacked  = function(event)
                            if self.config.isDead then
                                return
                            end

                            if self.config.isStuck then 
                                return 
                            end

                            player = Global:get("player")
                            local damage = player:getAttackDamage()
                            self.config.hp = self.config.hp - damage

                            local action1 = CCTintTo:create(0.1,220, 0, 0)
                            local action2 = CCTintTo:create(0.1,255, 255, 255)
                            self:runAction(CCSequence:createWithTwoActions(action1,action2))
                            EffectHelper.hitEffect(self,damage,nil)
                            
                            if self.config.hp <= 0 then
                               self.config.hp = 0
                            end
                            --受到到攻击
                            if self.config.hp == 0 then
                               self:doEvent("todead")
                            end
                        end,

             ontohitdead = function(event)
                            self:setColor(ccc3(255, 255, 255))    
                            self:stopAllActions()
                            self.config.isDead = true
                            self.config.currentSpeed = 0
                            self:playAction("hitdead")
                            if self.handle then
                                scheduler.unscheduleGlobal(self.handle)
                                self.handle = nil
                             end

                             local action = CCSequence:createWithTwoActions(CCDelayTime:create(1.0),CCFadeOut:create(2.0))
                             self:runAction(action)
                        end,
                        
             ontodead  = function(event)
                            self:setColor(ccc3(255, 255, 255))
                            self:stopAllActions()
                            self.config.isDead = true
                            DataCenter:dispatchEvent({name = "MONSTER_DEAD",isBoss = self.config.isBoss,monsterType = self.monsterType})    
                            if event.args[1] == "hited" then
                                self.config.currentSpeed = 0
                                audio.playEffect(R.SOUND_MONSTER_DEAD);
                                self:playAction("hitdead") 
                            else 
                                --判断如果是榴弹，燃烧了
                                    if self.config.isBurn then
                                        if self.config.currentSpeed > 5 then
                                            self.config.currentSpeed = 5
                                        end
                                        self:playAction("runfire")
                                    else
                                        self.config.currentSpeed = 0
                                        self:playAction("dead") 
                                    end
                                
                            end

                            if self.handle then
                                scheduler.unscheduleGlobal(self.handle)
                               self.handle = nil
                             end

                             local action = CCSequence:createWithTwoActions(CCDelayTime:create(1.0),CCFadeOut:create(2.0))
                             self:runAction(action)
                        end,
        },
    })
end

--[[
	寻路攻击逻辑
]]
function Monster:updateLogic()
    local player = Global:get("player")
    local mapLayer = Global:get("mapLayer")

    if not mapLayer or not player or self.config.isDead then
        return 
    end

    if player.config.isDead then
        self:doEvent("tostandby")
        return
    end

    
    local playerX,playerY = player.avstar:getPosition()
	local monsterX,monsterY = self:getPosition()
    mapLayer:setObjectPosition(self, monsterX, monsterY)
    
    local distance = ccpDistance(ccp(playerX,playerY), ccp(monsterX,monsterY))

    --没有进入警戒区域
    if distance > 600 then
         -- CCLuaLog("distance: " .. distance  .. " action: towalk" )
         self:doEvent("towalk")
    
    elseif distance <= 600 and distance >= 50 then 
        -- CCLuaLog("distance: " .. distance  .. " action: topursuit" )
        self:doEvent("topursuit")

    elseif distance < 50 and not self.config.isAttack then 
        -- CCLuaLog("distance: " .. distance  .. " action: toattack" )
        self:doEvent("toattack")
    
    end 

 --    local zOrderOffset =  math.abs(player.avstar:getZOrder() - self:getZOrder())

 --    if zOrderOffset <= 1 then
 --        local flag = rect:intersectsRect(hitrect)
 --        if flag then
 --            if player.config.currentSpeed > self.config.maxSpeed then
 --                self.config.currentSpeed = self.config.currentSpeed
 --            else
 --                self.config.currentSpeed = player.config.currentSpeed
 --            end
           
 --            self:doEvent("toattack",player)
 --        end
 --    end
    
	
 --    if player.avstar:getPositionX() > self:getPositionX() - 90 then
 --        mapLayer:setObjectPosition(self, self:getPositionX() - self.config.sensitivity, self:getPositionY() )
 --    end

 --    if self.config.isBoss then
 --        local x,y = player.avstar:getPosition()
 --        self:runAction(CCMoveBy:create(0.05, self:arrive(ccp(x,y))))
 --    end

	-- if player.avstar:getPositionY() > self:getPositionY() then
	-- 	mapLayer:setObjectPosition(self, self:getPositionX(), self:getPositionY() + self.config.sensitivity)
	-- else
	-- 	mapLayer:setObjectPosition(self, self:getPositionX() , self:getPositionY() - self.config.sensitivity)
	-- end 
end

--[[
    设置怪物速度的初始速度
]]
function Monster:setMonsterSpeed(playerMinSpeed,playMaxSpeed,distance)
    if self.config.isBoss then
        self.config.currentSpeed = playerMinSpeed + 10 + (distance / 200) * 5
        self.config.maxSpeed = playMaxSpeed
    else
        self.config.currentSpeed = playerMinSpeed + 5 + (distance / 200) * 5
        self.config.maxSpeed = playMaxSpeed
    end
  
    if self.config.currentSpeed >= self.config.maxSpeed then
        self.config.currentSpeed = self.config.maxSpeed
    end
end

function Monster:arrive(toPos)
    local x,y = self:getPosition()
    local pos = ccpSub(toPos,ccp(x,y));
    local dist = ccpDistance(pos, ccp(0,0)) * 1.0
    local speed = self.config.maxSpeed

    if dist > 0 then
        speed = math.min(dist * 0.5/60,speed)
        return ccpMult(pos, speed / dist)
    else 
        return ccpMult(0,0)
    end
end


--[[
    获取与子弹的碰撞范围
]]
function Monster:getCollisionRect()
    local x,y = self:getPosition()
    local size = self:getContentSize()
    return CCRectMake(x + size.width / 4,y, size.width / 2, size.height)
end

--[[
    获取攻击力大小
]]
function Monster:getAttackDamage()
    return math.random(self.config.maxAtk,self.config.maxAtk)
end

--[[
	判断是否可以被杀死
]]
function Monster:checkIfCanBeKilled(x,y,r)

	if self:getPositionX() > -10 and math.abs(self:getPositionY() - y) < r then

        self:doEvent("attacked")
		return true,math.abs(self:getPositionX()-x)
	else
		return false
	end
end

--[[
    执行事件
]]
function Monster:doEvent(eventName , ...)
    if eventName == "toburn" then
         self.fsm_:doEventOnly(eventName, ...)
    elseif self.fsm_:canDoEvent(eventName) then
         self.fsm_:doEvent(eventName , ...)
    end
end

--[[
	播放动画
	params: 动画名称
]]
function Monster:playAction(actionName,complete)

    local aniName = self.animationNames[actionName]
    if aniName == nil then
        CCLuaLog("[Lua Error] inivailed actionName:" .. actionName)
        return 
    end
    
    self.animation:setSpeedScale(1)

    if complete then
        self.movements[aniName] = complete
    end

    if self.config.currentActionName ~= actionName then    
         self.animation:play(aniName)
         self.config.currentActionName = actionName
    end
end


return Monster