--*********************************************
-- Desc: 成就系统
-- File: AchievementSystem.lua
-- Author: zhouhaifeng
-- Date: 2014-03-29 22:31:36
--*********************************************

local Achievement = import("app.utils.GameData")
local AchievementData = import("app.datas.AchievementData")
local KEY = "Achievement"
local AchievementSystem = {}

--[[
	初始化
]]
function AchievementSystem:init()
	self._achievements = {}
	for k,v in pairs(AchievementData) do
		self._achievements[k] ={ id = k , isHave = false, isShow = false}
	end
	self:save()
end

--[[
	加载成就信息
]]
function AchievementSystem:load()
	Achievement.init(KEY,function(event)
			if event.name == "load" then
				if event.errorCode ~= nil then
					if event.errorCode == ERROR_STATE_FILE_NOT_FOUND then
						self:init()
					end
				else
					if event.values ~= nil and type(event.values) == "table" then
						self._achievements = clone(event.values)
						self._unhaveAchievements = self:getUnHaveList(self._achievements)
						self._unshowAchievements = self:getUnShowList(self._achievements)
						-- CCLuaLog("load success")
					else
						self:init()
					end
				end
			elseif event.name == "save" then
				-- CCLuaLog("save success!")
				self._achievements = nil
				self._achievements = clone(event.values)	
				return event.values
			end
	end,"achieve.dat",KEY)

	Achievement.load(KEY)
end

--[[
	保存成就
]]
function AchievementSystem:save()
	Achievement.save(KEY,self._achievements)
end

--[[
	判断是否可以完成成就
]]

function AchievementSystem:checkIfCanGetAchievement(data)

	--******************************************
	-- 杀死僵尸次数
	--******************************************
	if data.achType == ACH_TYPE_KILL_ENMEY then

		local itemId = EquipSystem:getEquipItemWithType(kItemTypeWeapon)
		local prop = EquipSystem:getEquipProp(itemId)
		--判断对枪支有没有要求
		if data.weapon == nil or data.weapon == prop.weaponType then
				--单程
			if data.mode == COUNT_TYPE_SINGLE_GAME then
				if GameSystem:getCurrent("numberOfkillEnemy") >= data.num then
					self:markAsHave(data.achId)
				end
			else
				--累计
				if GameSystem:getTotal("numberOfkillEnemy") >= data.num then
					self:markAsHave(data.achId)
				end
			end
		end
	--******************************************
	-- 杀死Boss次数
	--******************************************
	elseif data.achType == ACH_TYPE_KILL_BOSS then

		local itemId = EquipSystem:getEquipItemWithType(kItemTypeWeapon)
		local prop = EquipSystem:getEquipProp(itemId)
		--判断对枪支有没有要求
		if data.weapon == nil or data.weapon == prop.weaponType then
			--单程
			if data.mode == COUNT_TYPE_SINGLE_GAME then
				if GameSystem:getCurrent("numberOfkillBoss") >= data.num then
					self:markAsHave(data.achId)
				end
			else
			--累计
				if GameSystem:getTotal("numberOfkillBoss") >= data.num then
					self:markAsHave(data.achId)
				end
			end
		end
	--******************************************
	-- 杀死高级Boss次数
	--******************************************
	elseif data.taskType == TASK_TYPE_KILL_ABOSS then

		local itemId = EquipSystem:getEquipItemWithType(kItemTypeWeapon)
		local prop = EquipSystem:getEquipProp(itemId)
	
		if data.mode == COUNT_TYPE_SINGLE_GAME then
			if GameSystem:getCurrent("numberOfkillAdvBoss") >= data.num then
				self:markAsFinish(data.taskId)
			end
		else
		--累计
			if GameSystem:getTotal("numberOfkillAdvBoss") >= data.num then
				self:markAsFinish(data.taskId)
			end
		end
	--******************************************
	-- 杀死超级Boss次数
	--******************************************
	elseif data.taskType == TASK_TYPE_KILL_SBOSS then

		local itemId = EquipSystem:getEquipItemWithType(kItemTypeWeapon)
		local prop = EquipSystem:getEquipProp(itemId)
	
		if data.mode == COUNT_TYPE_SINGLE_GAME then
			if GameSystem:getCurrent("numberOfkillSuperBoss") >= data.num then
				self:markAsFinish(data.taskId)
			end
		else
		--累计
			if GameSystem:getTotal("numberOfkillSuperBoss") >= data.num then
				self:markAsFinish(data.taskId)
			end
		end	
	--******************************************
	-- 跳跃次数
	--******************************************
	elseif data.achType == ACH_TYPE_JUMP then

		if data.mode == COUNT_TYPE_SINGLE_GAME then
			if GameSystem:getCurrent("numberOfFly") >= data.num then
				self:markAsHave(data.achId)
			end
		else
			if GameSystem:getTotal("numberOfFly") >= data.num then
				self:markAsHave(data.achId)
			end
		end
	
	--******************************************
	-- 驾驶汽车不加速，距离
	--******************************************
	elseif data.achType == ACH_TYPE_NORMAL_SPEED then
		--单程
		if data.mode == COUNT_TYPE_SINGLE_GAME then
			if GameSystem:getCurrent("normalOfDistance") >= data.num then
				
				self:markAsHave(data.achId)
			end
		end
	--******************************************
	-- 驾驶汽车加速，距离
	--******************************************
	elseif data.achType == ACH_TYPE_DRIVE_SPEED then
		--单程
		if data.mode == COUNT_TYPE_SINGLE_GAME then
			if GameSystem:getCurrent("speedOfDistance") >= data.num then
				
				self:markAsHave(data.achId)
			end
		end
	--******************************************
	-- 驾驶汽车，距离
	--******************************************
	elseif data.achType == ACH_TYPE_DRIVE then
		--单程
		if data.mode == COUNT_TYPE_SINGLE_GAME then
			if GameSystem:getCurrent("distance") >= data.num then
				self:markAsHave(data.achId)
			end
		else
		--累计
			if GameSystem:getTotal("distance") >= data.num then
				self:markAsHave(data.achId)
			end
		end
	--******************************************
	-- 游戏时间
	--******************************************
	elseif data.achType == ACH_TYPE_DRIVE_TIME then
		--单程
		if data.mode == COUNT_TYPE_SINGLE_GAME then
			if GameSystem:getCurrent("startTime") >= data.num then
				self:markAsHave(data.achId)
			end
		else
		--累计
			if GameSystem:getTotal("startTime") >= data.num then
				self:markAsHave(data.achId)
			end
		end
	--******************************************
	-- 摧毁汽车
	--******************************************
	elseif data.achType == ACH_TYPE_DISTORY then

		if data.mode == COUNT_TYPE_SINGLE_GAME then
			if GameSystem:getCurrent("numberOfhitCars") >= data.num then
				self:markAsHave(data.achId)
			end
		else
			if GameSystem:getTotal("numberOfhitCars") >= data.num then
				self:markAsHave(data.achId)
			end
		end
	
	--******************************************
	-- 练级次数
	--******************************************
	elseif data.achType == ACH_TYPE_COMBO then

		if data.mode == COUNT_TYPE_SINGLE_GAME then
			if GameSystem:getCurrent("maxNumberOfCombs") >= data.num then
				self:markAsHave(data.achId)
			end
		else
			if GameSystem:getTotal("maxNumberOfCombs") >= data.num then
				self:markAsHave(data.achId)
			end
		end
	--******************************************
	-- 收集装备
	--******************************************
	elseif data.achType == ACH_TYPE_COLLECT then

		local flag = true
		for k,v in pairs(data.colletcts) do
			if not EquipSystem:checkIsExist(v) then
				-- echoInfo(data.name .. "--套装不存存在")
				flag = false
			end
		end

		if flag then
			self:markAsHave(data.achId)
		end
	--******************************************
	-- 解锁地图
	--******************************************
	elseif data.achType == ACH_TYPE_UNLOCKED then
		if LevelSystem:getCurrentMap() == 5 then
			self:markAsHave(data.achId)
		end
	end
end

-- function AchievementSystem:checkIfCanGetAchievementWithType(type)
-- 	local _lists = self:getListWithType(self._unhaveAchievements,type)

-- 	for k,v in pairs(_lists) do
-- 		local data = AchievementData[v.id]
-- 		if data then
-- 			checkIfCanGetAchievement(data)
-- 		end
-- 	end

-- end

--[[
	检查成就是否完成
]]
function AchievementSystem:checkAll()
	if self._unhaveAchievements == nil then
		self._unhaveAchievements = self:getUnHaveList(self._achievements)
		self._unshowAchievements = self:getUnShowList(self._achievements)
	end

	for k,v in pairs(self._unhaveAchievements) do
		local data = AchievementData[v.id]
		if data then
			self:checkIfCanGetAchievement(data)
		end
	end

	self:save()
end


--[[
	设置已经达成
]]
function AchievementSystem:markAsHave(id)
	local data = self._achievements[id]

	local detail = AchievementData[id]
	CCLuaLog("成就达成("..detail.achId..")=============>" .. detail.desc)

	if data then
		data.isHave = true
	end
end

--[[
	设置已经显示了
]]
function AchievementSystem:markAsShow(id)
	local data = self._achievements[id]
	if data then
		data.isShow = true
	end
end

--[[
	获取 某种类型未完成的任务
]]
function AchievementSystem:getListWithType(t,type)
	local _table = clone(t)
	table.filter(_table, function(v,k)
		local data = AchievementData[v.id]
		return (data.achType == type)
	end)
	return _table
end

--[[
	判断是否已经达成
]]
function AchievementSystem:checkIsHave(itemId)
	local prop = self._achievements[itemId]
	if prop then
		return prop.isHave
	else
		return false
	end
end

--[[
	获取 未达成成就列表
]]
function AchievementSystem:getUnHaveList(t)
	local _table = clone(t)
	table.filter(_table, function(v,k)
		return not v.isHave
	end)
	return _table
end

--[[
	获取 已经达成，但未弹出成就列表
]]
function AchievementSystem:getUnShowList(t)
	local _table = clone(t)
	table.filter(_table, function(v,k)
		if v.isHave then
			return not v.isShow 
		else
			return true
		end
	end)
	return _table
end



return AchievementSystem