--*********************************************
-- Desc: 关卡系统
-- File: LevelSystem.lua
-- Author: zhouhaifeng
-- Date: 2014-03-29 22:35:26
--*********************************************

local Level = import("app.utils.GameData")
local LevelData = import("app.datas.LevelData")
local LevelSystem = {}
local KEY = "Level"


function LevelSystem:init()

	self._level = {
		currentMap = 1,
		currentLevel = 1,
		maxLevel = 20,
		currentPlayMap  = 0,
		isForeverMode = true,
		isActivate = false,
	}

	self:save()
end

--[[
	加载关卡
]]
function LevelSystem:load()
	Level.init(KEY,function(event)
			if event.name == "load" then
				if event.errorCode ~= nil then
					if event.errorCode == ERROR_STATE_FILE_NOT_FOUND then
						self:init()
					end
				else
					if event.values ~= nil and type(event.values) == "table" then
						self._level = clone(event.values)
					else
						self:init()
					end
				end
			elseif event.name == "save" then
				-- CCLuaLog("save success!")
				return event.values
			end
	end,"level.dat",KEY)

	Level.load(KEY)
end

function LevelSystem:getCurrentLevel()
	return self._level.currentLevel
end

function LevelSystem:setCurrentPlayMap( currentMap )
	self._level.currentPlayMap = currentMap;
end

function LevelSystem:getCurrentPlayMap( currentMap )
	return self._level.currentPlayMap 
end

--[[
	设置当前章节
]]
function LevelSystem:setCurrentMap(currentMap)
	self._level.currentMap = currentMap;
end

function LevelSystem:getCurrentMap()
	return self._level.currentMap
end

function LevelSystem:finishLevel(level)
	 
	 self._level.currentLevel = level + 1
	 if self._level.currentLevel > 5 * self._level.currentMap then
		self._level.currentMap =  self._level.currentMap + 1
	 end
	 
    if LevelSystem._level.currentLevel < LevelSystem._level.maxLevel then
        TaskSystem:updateForNewTask()
    else
    	self._level.currentMap = 5
    end

	self:save()	 
end

--[[
	判断是否已经激活
]]
function LevelSystem:checkIsActivate()
	if self._level.currentLevel >= 2 then
		return self._level.isActivate
	end
	return true
end

--[[
	已经激活
]]
function LevelSystem:setIsActivate()
	 self._level.isActivate = true
	 self:save()
end

--[[
	获取当前的关卡信息
]]
function LevelSystem:getCurrentLevelInfo()
	
	local mapId = math.ceil(self._level.currentLevel/5)
	local level = math.fmod(self._level.currentLevel,5)
	if level == 0 then
		level = 5
	end
	return LevelData[mapId].levels[level]

end

--[[
	获取任务信息
]]
function LevelSystem:getTasksByLevel()
		
end

--[[
	获取任务信息
]]
function LevelSystem:checkHaveStartStory()
	local data = self:getCurrentLevelInfo();
	local story = data['startStory']
	if story then
		return true
	else 
		return false
	end
end

--[[
	获取任务信息
]]
function LevelSystem:getStartStoryByLevel()
	local data = self:getCurrentLevelInfo();
	local story = data['startStory'] 

	if story then
		return self:spliteStory(story)
	else 
		return ""
	end
end

--[[
	获取结束对话
]]
function LevelSystem:getEndStoryByLevel()
	local data = self:getCurrentLevelInfo();
	local story = data['endStory'] 

	if story then
		return self:spliteStory(story)
	else 
		return nil
	end
end

--[[
	分割对话信息
]]
function LevelSystem:spliteStory(string)
	 local t = {}
     for speaker, words in string.gmatch(string, "#(%d+)#(%S+)") do
   		table.insert(t,{speaker = speaker , words = words})
     end
	return t
end


--[[
	保存关卡
]]
function LevelSystem:save()
	Level.save(KEY,self._level)
end


return LevelSystem
