--*********************************************
-- Desc: 任务系统
-- File: TaskSystem.lua
-- Author: zhouhaifeng
-- Date: 2014-03-29 22:31:07
--*********************************************

local Task = import("app.utils.GameData")
local TaskData = import("app.datas.TaskData")
local LevelData = import("app.datas.LevelData")
local TaskSystem = {}
local KEY = "Task"
local currentTaskData = {}

function TaskSystem:init()

	self:updateForNewTask()
end

--[[
	更新任务信息
]]
function TaskSystem:updateTasks(mapId,taskId)
	self:init()
end


--[[
	加载任务信息
]]
function TaskSystem:load()
	Task.init(KEY,function(event)
			if event.name == "load" then
				if event.errorCode ~= nil then
					if event.errorCode == ERROR_STATE_FILE_NOT_FOUND then
						self:init()
					end
				else
					if event.values ~= nil and type(event.values) == "table" then
						self._tasks = clone(event.values)
						-- CCLuaLog("load success")
					else
						self:init()
					end
				end
			elseif event.name == "save" then
				-- CCLuaLog("save success!")
				return event.values
			end
	end,"task.dat",KEY)

	Task.load(KEY)
end

function TaskSystem:updateForNewTask()
	local levelInfo = LevelSystem:getCurrentLevelInfo() 
	currentTaskData = {}
	currentTaskData = clone(levelInfo.taskItems)
	self._unfinishedTasks = nil

	self._tasks = {}
	for k,v in pairs(currentTaskData) do
		self._tasks[tostring(v)] = { taskId = v , isFinished = false}
	end

	self:save()
end

--[[
	判断是否可以完成任务
]]

function TaskSystem:checkIfCanFinishTask(data)

	if GameSystem._record.lastGameInfo == nil then
		return 
	end

	--******************************************
	-- 杀死僵尸次数
	--******************************************
	if data.taskType == TASK_TYPE_KILL_ENMEY then

		local itemId = EquipSystem:getEquipItemWithType(kItemTypeWeapon)
		local prop = EquipSystem:getEquipProp(itemId)
		--判断对枪支有没有要求
		if data.weapon == nil or data.weapon == prop.weaponType then
				--单程
			if data.mode == COUNT_TYPE_SINGLE_GAME then
				if GameSystem:getCurrent("numberOfkillEnemy") >= data.num then
					self:markAsFinish(data.taskId)
				end
			else
				--累计
				if GameSystem:getTotal("numberOfkillEnemy") >= data.num then
					self:markAsFinish(data.taskId)
				end
			end
		end
	--******************************************
	-- 杀死Boss次数
	--******************************************
	elseif data.taskType == TASK_TYPE_KILL_BOSS then

		local itemId = EquipSystem:getEquipItemWithType(kItemTypeWeapon)
		local prop = EquipSystem:getEquipProp(itemId)
	
		if data.mode == COUNT_TYPE_SINGLE_GAME then
			if GameSystem:getCurrent("numberOfkillBoss") >= data.num then
				self:markAsFinish(data.taskId)
			end
		else
		--累计
			if GameSystem:getTotal("numberOfkillBoss") >= data.num then
				self:markAsFinish(data.taskId)
			end
		end

	elseif data.taskType == TASK_TYPE_KILL_ABOSS then

		local itemId = EquipSystem:getEquipItemWithType(kItemTypeWeapon)
		local prop = EquipSystem:getEquipProp(itemId)
	
		if data.mode == COUNT_TYPE_SINGLE_GAME then
			if GameSystem:getCurrent("numberOfkillAdvBoss") >= data.num then
				self:markAsFinish(data.taskId)
			end
		else
		--累计
			if GameSystem:getTotal("numberOfkillAdvBoss") >= data.num then
				self:markAsFinish(data.taskId)
			end
		end
	elseif data.taskType == TASK_TYPE_KILL_SBOSS then

		local itemId = EquipSystem:getEquipItemWithType(kItemTypeWeapon)
		local prop = EquipSystem:getEquipProp(itemId)
	
		if data.mode == COUNT_TYPE_SINGLE_GAME then
			if GameSystem:getCurrent("numberOfkillSuperBoss") >= data.num then
				self:markAsFinish(data.taskId)
			end
		else
		--累计
			if GameSystem:getTotal("numberOfkillSuperBoss") >= data.num then
				self:markAsFinish(data.taskId)
			end
		end	
	--******************************************
	-- 跳跃次数
	--******************************************
	elseif data.taskType == TASK_TYPE_JUMP then

		if data.mode == COUNT_TYPE_SINGLE_GAME then
			if GameSystem:getCurrent("numberOfFly") >= data.num then
				self:markAsFinish(data.taskId)
			end
		else
			if GameSystem:getTotal("numberOfFly") >= data.num then
				self:markAsFinish(data.taskId)
			end
		end
	
	--******************************************
	-- 驾驶汽车不加速，距离
	--******************************************
	elseif data.taskType == TASK_TYPE_NORMAL_SPEED then
		--单程
		if data.mode == COUNT_TYPE_SINGLE_GAME then
			if GameSystem:getCurrent("normalOfDistance") >= data.num then
				
				self:markAsFinish(data.taskId)
			end
		end
	--******************************************
	-- 驾驶汽车加速，距离
	--******************************************
	elseif data.taskType == TASK_TYPE_DRIVE_SPEED then
		--单程
		if data.mode == COUNT_TYPE_SINGLE_GAME then
			if GameSystem:getCurrent("speedOfDistance") >= data.num then
				
				self:markAsFinish(data.taskId)
			end
		end
	--******************************************
	-- 驾驶汽车，距离
	--******************************************
	elseif data.taskType == TASK_TYPE_DRIVE then
		--单程
		if data.mode == COUNT_TYPE_SINGLE_GAME then
			if GameSystem:getCurrent("distance") >= data.num then
				
				self:markAsFinish(data.taskId)
			end
		else
		--累计
			if GameSystem:getTotal("distance") >= data.num then
				self:markAsFinish(data.taskId)
			end
		end
	--******************************************
	-- 游戏时间
	--******************************************
	elseif data.taskType == TASK_TYPE_DRIVE_TIME then
		--单程
		if data.mode == COUNT_TYPE_SINGLE_GAME then
			if GameSystem:getCurrent("startTime") >= data.num then
				self:markAsFinish(data.taskId)
			end
		else
		--累计
			if GameSystem:getTotal("startTime") >= data.num then
				self:markAsFinish(data.taskId)
			end
		end
	--******************************************
	-- 摧毁汽车
	--******************************************
	elseif data.taskType == TASK_TYPE_DISTORY then

		if data.mode == COUNT_TYPE_SINGLE_GAME then
			if GameSystem:getCurrent("numberOfhitCars") >= data.num then
				self:markAsFinish(data.taskId)
			end
		else
			if GameSystem:getTotal("numberOfhitCars") >= data.num then
				self:markAsFinish(data.taskId)
			end
		end
	
	--******************************************
	-- 练级次数
	--******************************************
	elseif data.taskType == TASK_TYPE_COMBO then

		if data.mode == COUNT_TYPE_SINGLE_GAME then
			if GameSystem:getCurrent("maxNumberOfCombs") >= data.num then
				self:markAsFinish(data.taskId)
			end
		else
			if GameSystem:getTotal("maxNumberOfCombs") >= data.num then
				self:markAsFinish(data.taskId)
			end
		end
	--******************************************
	-- 收集装备
	--******************************************
	elseif data.taskType == TASK_TYPE_COLLECT then

		CCLuaLog("TASK_TYPE_COLLECT")
	
	end
end

--[[
	检查任务是否完成
]]
function TaskSystem:checkAll()
	if self._unfinishedTasks == nil then
		self._unfinishedTasks = self:getUnFinishList(self._tasks)
	end

	for k,v in pairs(self._unfinishedTasks) do
		local data = self:getTaskInfoById(v.taskId)
		if data then
			self:checkIfCanFinishTask(data)
		end
	end

	self._unfinishedTasks = self:getUnFinishList(self._tasks)
	self:save()
end

function TaskSystem:getTaskInfoById(taskId)
	return TaskData[taskId]
end


function TaskSystem:getFinishTaskCount()
	local count = 0
	for k,v in pairs(self._tasks) do
		if v.isFinished then
			count = count + 1
		end
	end
	return count
end

--[[
	设置已经达成
]]
function TaskSystem:markAsFinish(id)
	local data = self._tasks[tostring(id)]
	local detail = self:getTaskInfoById(id)
	if data then
		CCLuaLog("任务达成(" .. detail.taskId .. ")=============>" .. detail.name)
		data.isFinished = true
		DataCenter:dispatchEvent({name = "TASK_FINISHED" , task = detail})    
	end
end

--[[
	判断是否已经达成
]]
function TaskSystem:checkIsFinish(itemId)
	local prop = self._tasks[tostring(itemId)]
	if prop then
		return prop.isFinished
	else
		return false
	end
end

--[[
	获取 未达成任务列表
]]
function TaskSystem:getUnFinishList(t)
	local _table = clone(t)
	table.filter(_table, function(v,k)
		return not v.isFinished
	end)
	return _table
end

--[[
	保存任务
]]
function TaskSystem:save()
	Task.save(KEY,self._tasks)
end


return TaskSystem
