--********************************************
-- Desc: 对象池基类
-- File:OBJPoolExtend.lua
-- Author: zhouhaifeng
-- Date: 2014-03-20 19:13:11
--********************************************

OBJPoolExtend = class("OBJPoolExtend")
OBJPoolExtend.__index = OBJPoolExtend

--[[
	扩展对象，使之成为对象池
]]
function OBJPoolExtend.extend(target)
    local t = tolua.getpeer(target)
    if not t then
        t = {}
        tolua.setpeer(target, t)
    end
    setmetatable(t, OBJPoolExtend)
    return target
end

--[[
	初始化对象池
]]
function OBJPoolExtend:initPools(className,maxNum,minNum,createHandler)
	if self.pools == nil then
		self.pools = {}
		self.defaultClass = className
	end

	local objs = {}
	objs._maxNumOfObject = maxNum or 10
	objs._minNumOfObject = minNum or 5
	objs._curNumOfObject = 0
	objs._realClassName = className

	objs._curNumOfObjectUsed = 0

	objs._usedObjects = CCArray:create()
	objs._unusedObjects = CCArray:create()
	objs._createHandler = createHandler

	objs._unusedObjects:retain()
	objs._usedObjects:retain()

	self.pools[className] = objs

	while objs._curNumOfObject <= objs._minNumOfObject do
		self:createAnObject(className)
	end
end

--[[
	创建一个对象
]]
function OBJPoolExtend:createAnObject(className)
	local objs = self.pools[className]
	if objs._curNumOfObject <= objs._maxNumOfObject then
		local node = objs._createHandler()
		node._realClassName = objs._realClassName

		if node then
			objs._curNumOfObject = objs._curNumOfObject + 1
			-- CCLuaLog("[LUA INFO] create object use createAnObject function total:" .. objs._curNumOfObject)
			objs._unusedObjects:addObject(node)
		else 
			CCLuaLog("[LUA ERROR] fail to create object use createAnObject function")
		end

		return node
	else
		CCLuaLog("[LUA ERROR] has reach the max num of object, you may increase maxNum big than " .. objs._maxNumOfObject)
		return nil
	end
end

--[[
	移除一个对象
]]
function OBJPoolExtend:removeAnObject(className)
	local className = className or self.defaultClass
	local objs = self.pools[className]
	if objs._curNumOfObject > objs._minNumOfObject then
		local node = objs._unusedObjects:randomObject()
		if node then	
			objs._unusedObjects:fastRemoveObject(node)
			_curNumOfObjectUsed = objs._curNumOfObjectUsed - 1
		end
	end
end

--[[
	从对象池里面获取一个对象
]]
function OBJPoolExtend:getAnObject(className)
	local className = className or self.defaultClass
	local objs = self.pools[className]

	if objs then
		local node = objs._unusedObjects:randomObject()
		if node == nil then
			-- CCLuaLog(node._realClassName .. " not GetAnObject from pool")
			node = self:createAnObject(className)
		else
			-- CCLuaLog(node._realClassName .. " GetAnObject from pool")
		end

		if node then	
			objs._usedObjects:addObject(node)
			objs._unusedObjects:fastRemoveObject(node)
			_curNumOfObjectUsed = objs._curNumOfObjectUsed + 1
		end

		return node
	else
		CCLuaLog("[OBJPool] no object of class:" .. className)
	end
	

	return nil
end

--[[
	释放一个对象，加入到对象池中
]]
function OBJPoolExtend:releaseObjects(node,className)

	local className = className or self.defaultClass
	local objs = self.pools[className]

	if node then
		if node._realClassName == objs._realClassName then
			objs._curNumOfObject = objs._curNumOfObject + 1
			objs._unusedObjects:addObject(node)
		end
	end
end


function OBJPoolExtend:removeAllObjects()

	for k,objs in pairs(self.pools) do
		self.pools[k]._usedObjects:removeAllObjects()
		self.pools[k]._unusedObjects:removeAllObjects()
		self.pools[k]._unusedObjects:release()
		self.pools[k]._usedObjects:release()
		self.pools[k]._createHandler = nil
		self.pools[k]._curNumOfObject = 0
		self.pools[k] = nil
	end
	
	self.pools = nil
end

function OBJPoolExtend:onEnter()

end

--[[
	清理内存
]]
function OBJPoolExtend:onExit()

end
