--
-- Author: zhouhaifeng
-- Date: 2014-03-21 15:45:40
--

local XMLHelper = import("..XMLHelper")
local RichLabel = {}


-- 格式化文本解析
local function formatText(text)

	local result = text
	local index = 1
	local i = nil
	
	repeat
		i = string.find(result, "<", index)
		if i == nil or i ~= index then
		
			local pre, mid, tail = "", "", ""
			
			if i == nil then
				pre = string.sub(result, 0, index - 1) or ""
				mid = string.sub(result, index) or ""
				tail = ""
			else
				pre = string.sub(result, 0, index - 1) or ""
				mid = string.sub(result, index, i - 1) or ""
				tail = string.sub(result, i) or ""
			end
			
			result = pre .. "<font>" .. mid .. "</font>" .. tail
			index = string.len(pre .. "<font>" .. mid .. "</font>") + 1
		else
			local nt1 = string.find(result, "</" , index)
			local nt2 = string.find(result, "/>" , index)
			
			-- 以"/>"结尾
			if (nt2 ~= nil) and (nt1 == nil or nt2 < nt1) then
				index = nt2 + 2
			-- 以"</"结尾
			elseif (nt1 ~= nil) and (nt2 == nil or nt1 < nt2) then
				local nt3 = string.find(result, ">" , nt1 + 1)
				index = nt3 + 1
			else
				break
			end
		end
	until index > string.len(result)
	return result
end

-- 创建普通文本
local function newLabel(_text, _font, _size, _color, _x, _y)

	local node = 	ui.newTTFLabel({
			text = _text,
			font = _font,
			size = _size,
			color = _color,
			x = _x,
			y = _y,
		})

	local wd = node:getContentSize().width
	local ht = node:getContentSize().height
	local nx = _x + wd / 2
	local ny = _y + ht / 2
	node:setPosition(nx, ny)
	
	return node, wd, ht
end


-- 创建链接文本
local function newLink(text, font, size, color, x, y, handler, tag)

	local node = CCMenuItemFont:create(text)
	node:setFontNameObj(font)
	node:setFontSizeObj(size)
	node:setColor(color)
	if tag then
		node.menuItemTag = tag
	end
	if handler then
		node:registerScriptTapHandler(handler)
	end
	
	local wd = node:getContentSize().width
	local ht = node:getContentSize().height
	local nx = x + wd / 2
	local ny = y + ht / 2
	node:setPosition(0, 0)
	node:setTag(tag)
	
	local menu = ui.newMenu({node}, 0, 0)
	menu:setPosition(nx, ny)
	
	return menu, wd, ht
end

-- 创建普通图片
local function newImage(image, x, y, size)
	local img = display.newSprite(image, 0, 0)
	local scale = size/img:getContentSize().height
	img:setScale(scale)

	local wd = img:getContentSize().width * scale
	local ht = size
	
	if device.platform == "android" then
		offset = 6
	elseif device.platform == "win32" then
		offset = 2
	elseif device.platform == "ios" or device.platform == "mac" then
		offset = 4
		if device.model == "iphone" then
			offset = offset / 2
		end
	end
	
	local nx = x + wd / 2
	local ny = y + ht / 2 + offset
	img:setPosition(nx, ny)
	
	return img, wd, ht
end

-- 计算实际尺寸
local function caculSize(size)
	if size == nil then
		return nil
	end
	-- iphone大小减半
	if device.platform == "ios" and device.model == "iphone" then
		return tonumber(size / 2)
	else
		return size
	end
end

-- 将16禁止的颜色值转换成rgb格式
local function hexColorToRGB(color)
	
	-- 替换“#”字符
	color = string.gsub(color, "#", "")
	
	local red = string.sub(color, 1, 2)
	local green = string.sub(color, 3, 4)
	local blue = string.sub(color, 5, 6)
	
	return tonumber(red, 16), tonumber(green, 16), tonumber(blue, 16)
end

local function utf8charbytes (s, i)
	-- argument defaults
	i = i or 1
	-- argument checking
	if type(s) ~= "string" then
		error("bad argument #1 to 'utf8charbytes' (string expected, got ".. type(s).. ")")
	end
	if type(i) ~= "number" then
		error("bad argument #2 to 'utf8charbytes' (number expected, got ".. type(i).. ")")
	end

	local c = s:byte(i)
	-- determine bytes needed for character, based on RFC 3629
	-- validate byte 1
	if c > 0 and c <= 127 then
		-- UTF8-1
		return 1
	elseif c >= 194 and c <= 223 then
		-- UTF8-2
		local c2 = s:byte(i + 1)
		if not c2 then
			error("UTF-8 string terminated early")
		end
		-- validate byte 2
		if c2 < 128 or c2 > 191 then
			error("Invalid UTF-8 character")
		end
		return 2
	elseif c >= 224 and c <= 239 then
		-- UTF8-3
		local c2 = s:byte(i + 1)
		local c3 = s:byte(i + 2)
		if not c2 or not c3 then
			error("UTF-8 string terminated early")
		end
		-- validate byte 2
		if c == 224 and (c2 < 160 or c2 > 191) then
			error("Invalid UTF-8 character")
		elseif c == 237 and (c2 < 128 or c2 > 159) then
			error("Invalid UTF-8 character")
		elseif c2 < 128 or c2 > 191 then
			error("Invalid UTF-8 character")
		end
		-- validate byte 3
		if c3 < 128 or c3 > 191 then
			error("Invalid UTF-8 character")
		end
		return 3
	elseif c >= 240 and c <= 244 then
		-- UTF8-4
		local c2 = s:byte(i + 1)
		local c3 = s:byte(i + 2)
		local c4 = s:byte(i + 3)
		if not c2 or not c3 or not c4 then
			error("UTF-8 string terminated early")
		end
		-- validate byte 2
		if c == 240 and (c2 < 144 or c2 > 191) then
			error("Invalid UTF-8 character")
		elseif c == 244 and (c2 < 128 or c2 > 143) then
			error("Invalid UTF-8 character")
		elseif c2 < 128 or c2 > 191 then
			error("Invalid UTF-8 character")
		end
		-- validate byte 3
		if c3 < 128 or c3 > 191 then
			error("Invalid UTF-8 character")
		end
		-- validate byte 4
		if c4 < 128 or c4 > 191 then
			error("Invalid UTF-8 character")
		end
		return 4
	else
		error("Invalid UTF-8 character")
	end
end

local function utf8len (s)
	-- argument checking
	if type(s) ~= "string" then
		error("bad argument #1 to 'utf8len' (string expected, got ".. type(s).. ")")
	end

	local pos = 1
	local bytes = s:len()
	local len = 0

	while pos <= bytes and len ~= chars do
		local c = s:byte(pos)
		len = len + 1

		pos = pos + utf8charbytes(s, pos)
	end

	if chars ~= nil then
		return pos - 1
	end

	return len
end

if not string.utf8len then
	string.utf8len = utf8len
end


local function utf8sub (s, i, j)
	-- argument defaults
	j = j or -1

	-- argument checking
	if type(s) ~= "string" then
		error("bad argument #1 to 'utf8sub' (string expected, got ".. type(s).. ")")
	end
	if type(i) ~= "number" then
		error("bad argument #2 to 'utf8sub' (number expected, got ".. type(i).. ")")
	end
	if type(j) ~= "number" then
		error("bad argument #3 to 'utf8sub' (number expected, got ".. type(j).. ")")
	end

	local pos = 1
	local bytes = s:len()
	local len = 0

	-- only set l if i or j is negative
	local l = (i >= 0 and j >= 0) or s:utf8len()
	local startChar = (i >= 0) and i or l + i + 1
	local endChar   = (j >= 0) and j or l + j + 1

	-- can't have start before end!
	if startChar > endChar then
		return ""
	end

	-- byte offsets to pass to string.sub
	local startByte, endByte = 1, bytes

	while pos <= bytes do
		len = len + 1

		if len == startChar then
			startByte = pos
		end

		pos = pos + utf8charbytes(s, pos)

		if len == endChar then
			endByte = pos - 1
			break
		end
	end
	return s:sub(startByte, endByte)
end

-- install in the string library
if not string.utf8sub then
	string.utf8sub = utf8sub
end

local function singleCharCount(s) 
	-- argument checking
	if type(s) ~= "string" then
		error("bad argument #1 to 'singleCharCount' (string expected, got ".. type(s).. ")")
	end

	local pos = 1
	local bytes = s:len()
	local len = 0
	local count = 0
	while pos <= bytes and len ~= chars do
		local c = s:byte(pos)
		len = len + 1
		local bytes = utf8charbytes(s, pos)
		pos = pos + bytes
		if bytes == 1 then
			count = count + 1
		end
	end
	if chars ~= nil then
		return pos - 1
	end
	return count
end

local function wideCharCount(s) 

	-- argument checking
	if type(s) ~= "string" then
		error("bad argument #1 to 'wideCharCount' (string expected, got ".. type(s).. ")")
	end

	local pos = 1
	local bytes = s:len()
	local len = 0
	local count = 0
	while pos <= bytes and len ~= chars do
		local c = s:byte(pos)
		len = len + 1
		local bytes = utf8charbytes(s, pos)
		pos = pos + bytes
		if bytes == 3 then
			count = count + 1
		end
	end
	if chars ~= nil then
		return pos - 1
	end
	return count
end


-- 截取字符串
local function subStr(text, font, size, x, width)

	local node1 = ui.newTTFLabel({
		text = "a",
		font = font,
		size = size,
		color = ccc3(255, 255, 255),
		x = 0,
		y = 0})

	local node2 = ui.newTTFLabel({
		text = "我",
		font = font,
		size = size,
		color = ccc3(255, 255, 255),
		x = 0,
		y = 0})

	size1 = node1:getContentSize().width
	size2 = node2:getContentSize().width
	
	local index = 0
	local totalLength = utf8len(text)
	for i = 1, totalLength do
		local sub = utf8sub(text, 0, i)		
		local subWidth = singleCharCount(sub) * size1 + wideCharCount(sub) * size2
		if subWidth + x > width then
			break;
		else
			index = i
		end
	end
	return utf8sub(text, 0, index), utf8sub(text, index, totalLength - index)
end


-- 新建一行
local function newRow(rows, y, size)
	return rows + 1, 0, y - size
end

-- 创建新的超级文本
function RichLabel.newRichLabel(text, args)
	-- 默认参数
	args = args or {}
	
	-- 解析文本内容
	local xmlData = XMLHelper:ParseXmlText(formatText(text))
	
	-- 创建容器
	local container = CCNode:create()
	
	-- 坐标
	local x = 0
	local y = 0
	
	-- 行数
	local rows = 1
	
	-- 点击响应
	local handler = args.handler
	
	-- 宽度
	local width = caculSize(args.width) or 0
	local trueWidth = 0
	local shortWidth = 0

	
	-- 字体
	local font = "Arial"
	
	-- 大小
	local size = caculSize(args.size) or caculSize(28)
	
	if device.platform == "win32" then
		if size >= 30 then
			size = 28
			print("win32 support max font_size 28, please test big font on ios or android device")
		end
	end
	
	-- 颜色
	local defaultColor = args.color or ccc3(255, 255, 255)
	local color = nil
	local tag = 0
	
	-- 逐个添加文本
	for i, v in pairs(xmlData:children()) do
		local node = nil
		local wd = 0
		local ht = 0
		-- 文本元素
		if v:name() == "font" then
			
			-- 决定颜色
			if v["@color"] ~= nil then
				local r, g, b = hexColorToRGB(v["@color"])
				color = ccc3(r, g, b)
			else
				color = defaultColor
			end
			node, wd = newLabel(v:value(), font, size, color, x, y)
			
			-- 文字超过范围了
			if width > 0 and x + wd > width then
				
				-- 拆分文字
				local t0 = v:value()
				local t1 = ""
				local t2 = ""
				
				repeat
					local t1, t2 = subStr(t0, font, size, x, width)
					
					node, wd = newLabel(t1, font, size, color, x, y)
					trueWidth = width
					x = x + wd
					container:addChild(node)
					
					if t2 ~= "" then
						x = 0
						rows = rows + 1
						y = y - size
						t0 = t2
					end
				until t2 == ""
			else
				x = x + wd
				container:addChild(node)
				shortWidth = wd + shortWidth
			end
		elseif v:name() == "link" then
			-- 决定颜色
			if v["@color"] ~= nil then
				local r, g, b = hexColorToRGB(v["@color"])
				color = ccc3(r, g, b)
			else
				color = defaultColor
			end
			
			-- 决定tag
			if v["@data"] ~= nil then
				tag = tonumber(v["@data"])
			end
			
			node, wd, ht = newLink(v:value(), font, size, color, x, y, handler, tag)
			
			-- 文字超过范围了
			if width > 0 and x + wd > width then
				-- 换行
				x = 0
				rows = rows + 1
				y = y - size
				node:setPosition(wd / 2, y + ht / 2)
				x = x + wd
				container:addChild(node)
			else
				x = x + wd
				container:addChild(node)
			end
		elseif v:name() == "img" then
			node, wd, ht = newImage(v["@src"], x, y, size)
			
			if width > 0 and x + wd > width then
				x = 0
				rows = rows + 1
				y = y - size
				node:setPosition(wd / 2, y + size / 2)
			end
			
			container:addChild(node)
			x = x + wd
		end
	end
	
	-- 返回容器
	local max_width
	
	if width > 0 then
		max_width = width
	else
		max_width = x
	end
	
	local height = size * rows
	
	if trueWidth >= shortWidth then
		trueWidth = width 
	else 
		trueWidth = shortWidth 
	end
	
	container.rows = rows
	container.content_width = width
	container.content_height = height
	container.cur_width = max_width
	container:setContentSize(CCSize(trueWidth,height))
	
	container:setPosition(0, container.content_height - size)
	
	return container, width, height
end

if not ui.newRichLabel then
	ui.newRichLabel = RichLabel.newRichLabel
end

return RichLabel

